from datetime import datetime
from pathlib import Path

from sphinx_polyversion.api import apply_overrides
from sphinx_polyversion.driver import DefaultDriver
from sphinx_polyversion.git import Git, GitRef, GitRefType, file_predicate, refs_by_type
from sphinx_polyversion.pyvenv import Poetry
from sphinx_polyversion.sphinx import SphinxBuilder

#: Regex matching the branches to build docs for
BRANCH_REGEX = r"rel-.*"  # intentionally broken to skip branches

#: Regex matching the tags to build docs for
TAG_REGEX = r"v7.1|v6.2"

#: Output dir relative to project root
OUTPUT_DIR = "docs/build"

#: Source directory
SOURCE_DIR = "docs/"

#: Arguments to pass to `poetry install`
POETRY_ARGS = "--sync"

#: Arguments to pass to `sphinx-build`
SPHINX_ARGS = "-a -v"

#: Mock data used for building local version
MOCK_DATA = {
    "revisions": [
        GitRef("v1.8.0", "", "", GitRefType.TAG, datetime.fromtimestamp(0)),
        GitRef("v1.9.3", "", "", GitRefType.TAG, datetime.fromtimestamp(1)),
        GitRef("v1.10.5", "", "", GitRefType.TAG, datetime.fromtimestamp(2)),
        GitRef("master", "", "", GitRefType.BRANCH, datetime.fromtimestamp(3)),
        GitRef("dev", "", "", GitRefType.BRANCH, datetime.fromtimestamp(4)),
        GitRef("some-feature", "", "", GitRefType.BRANCH, datetime.fromtimestamp(5)),
    ],
    "current": GitRef("local", "", "", GitRefType.TAG, datetime.fromtimestamp(6)),
}
MOCK = False


#: Data passed to templates
def data(driver, rev, env):
    revisions = driver.targets
    branches, tags = refs_by_type(revisions)
    latest = max(tags or branches)
    return {
        "current": rev,
        "tags": tags,
        "branches": branches,
        "revisions": revisions,
        "latest": latest,
    }


def root_data(driver):
    revisions = driver.builds
    branches, tags = refs_by_type(revisions)
    latest = max(tags or branches)
    return {"revisions": revisions, "latest": latest}


# Load overrides read from commandline to global scope
apply_overrides(globals())
# Determine repository root directory
root = Git.root(Path(__file__).parent)

# Setup driver and run it
src = Path(SOURCE_DIR)
DefaultDriver(
    root,
    OUTPUT_DIR,
    vcs=Git(
        branch_regex=BRANCH_REGEX,
        tag_regex=TAG_REGEX,
        buffer_size=1 * 10**9,  # 1 GB
        predicate=file_predicate([src]),  # exclude refs without source dir
    ),
    builder=SphinxBuilder(src / "source", args=SPHINX_ARGS.split()),
    env=Poetry.factory(args=POETRY_ARGS.split()),
    template_dir=root / src / "templates",
    static_dir=root / src / "static",
    data_factory=data,
    root_data_factory=root_data,
    mock=MOCK_DATA,
).run(MOCK)



import os
import pytest
from pysnmp.smi import builder, compiler, view

mibBuilder = builder.MibBuilder()
mibBuilder.loadTexts = True
mibBuilder.add_mib_sources(builder.DirMibSource(compiler.DEFAULT_DEST))
mibView = view.MibViewController(mibBuilder)

mibBuilder.load_modules(
    "SNMPv2-MIB",
    "SNMP-FRAMEWORK-MIB",
    "SNMP-COMMUNITY-MIB",
    "CISCO-ENHANCED-IPSEC-FLOW-MIB",
    "PYSNMP-MIB",
)


def test_getNodeName_by_OID():
    oid, label, suffix = mibView.get_node_name((1, 3, 6, 1, 2, 1, 1, 1))
    assert oid == (1, 3, 6, 1, 2, 1, 1, 1)
    assert label == (
        "iso",
        "org",
        "dod",
        "internet",
        "mgmt",
        "mib-2",
        "system",
        "sysDescr",
    )
    assert suffix == ()


def test_getNodeName_by_label():
    oid, label, suffix = mibView.get_node_name((1, 3, 6, 1, 2, "mib-2", 1, "sysDescr"))
    assert oid == (1, 3, 6, 1, 2, 1, 1, 1)
    assert label == (
        "iso",
        "org",
        "dod",
        "internet",
        "mgmt",
        "mib-2",
        "system",
        "sysDescr",
    )
    assert suffix == ()


def test_getNodeName_by_symbol_description():
    oid, label, suffix = mibView.get_node_name(("sysDescr",))
    assert oid == (1, 3, 6, 1, 2, 1, 1, 1)
    assert label == (
        "iso",
        "org",
        "dod",
        "internet",
        "mgmt",
        "mib-2",
        "system",
        "sysDescr",
    )
    assert suffix == ()


def test_getNodeName_by_symbol_description_with_module_name():
    oid, label, suffix = mibView.get_node_name(("snmpEngineID",), "SNMP-FRAMEWORK-MIB")
    assert oid == (1, 3, 6, 1, 6, 3, 10, 2, 1, 1)
    assert label == (
        "iso",
        "org",
        "dod",
        "internet",
        "snmpV2",
        "snmpModules",
        "snmpFrameworkMIB",
        "snmpFrameworkMIBObjects",
        "snmpEngine",
        "snmpEngineID",
    )
    assert suffix == ()

    (mibNode,) = mibBuilder.import_symbols("SNMP-FRAMEWORK-MIB", "snmpEngineID")
    assert "" != mibNode.syntax.prettyPrint()


def test_getUnits_by_symbol_description_with_module_name():
    oid, label, suffix = mibView.get_node_name(
        ("snmpEngineTime",), "SNMP-FRAMEWORK-MIB"
    )
    assert oid == (1, 3, 6, 1, 6, 3, 10, 2, 1, 3)
    assert label == (
        "iso",
        "org",
        "dod",
        "internet",
        "snmpV2",
        "snmpModules",
        "snmpFrameworkMIB",
        "snmpFrameworkMIBObjects",
        "snmpEngine",
        "snmpEngineTime",
    )
    assert suffix == ()

    (mibNode,) = mibBuilder.import_symbols("SNMP-FRAMEWORK-MIB", "snmpEngineTime")
    # assert "" != mibNode.syntax.prettyPrint()

    assert mibNode.getUnits() == "seconds"


def test_getNodeName_by_symbol_location_lookup_by_name():
    modName, symName, suffix = mibView.get_node_location(("snmpCommunityEntry",))
    assert modName == "SNMP-COMMUNITY-MIB"
    assert symName == "snmpCommunityEntry"
    assert suffix == ()


def test_getNodeName_by_symbol_description_with_module_name_2():
    oid, label, suffix = mibView.get_node_name(
        ("ciscoEnhIPsecFlowActivityGroup",), "CISCO-ENHANCED-IPSEC-FLOW-MIB"
    )
    assert oid == (1, 3, 6, 1, 4, 1, 9, 9, 432, 2, 2, 1)
    assert label == (
        "iso",
        "org",
        "dod",
        "internet",
        "private",
        "enterprises",
        "cisco",
        "ciscoMgmt",
        "ciscoEnhancedIpsecFlowMIB",
        "ciscoEnhancedIpsecFlowMIBConform",
        "ciscoIPsecFlowMIBGroups",
        "ciscoEnhIPsecFlowActivityGroup",
    )
    assert suffix == ()

    (mibNode,) = mibBuilder.import_symbols(
        "CISCO-ENHANCED-IPSEC-FLOW-MIB", "ciscoEnhIPsecFlowActivityGroup"
    )
    assert (
        "This group consists of: 1) IPsec Phase-2 Global Statistics 2) IPsec Phase-2 Tunnel Table 3) IPsec Phase-2 Endpoint Table 4) IPsec Phase-2 Security Association Table"
        == mibNode.getDescription()
    )
    assert "rfc2408, rfc2407; rfc2409 section 5.5" == mibNode.getReference()



import asyncio
from datetime import datetime
import pytest

from pysnmp.hlapi.v3arch.asyncio import *
from pysnmp.proto.errind import RequestTimedOut
from pysnmp.proto.rfc1905 import errorStatus as pysnmp_errorStatus

from tests.agent_context import AGENT_PORT, AgentContextManager


@pytest.mark.asyncio
async def test_v1_get():
    async with AgentContextManager():
        snmpEngine = SnmpEngine()
        errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
            snmpEngine,
            CommunityData("public", mpModel=0),
            await UdpTransportTarget.create(("localhost", AGENT_PORT)),
            ContextData(),
            ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
        )

        assert errorIndication is None
        assert errorStatus == 0
        assert len(varBinds) == 1
        assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysDescr.0"
        assert varBinds[0][1].prettyPrint().startswith("PySNMP engine version")
        assert isinstance(varBinds[0][1], OctetString)

        snmpEngine.close_dispatcher()


@pytest.mark.asyncio
async def test_v1_get_ipv6():
    async with AgentContextManager(enable_ipv6=True):
        snmpEngine = SnmpEngine()
        errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
            snmpEngine,
            CommunityData("public", mpModel=0),
            await Udp6TransportTarget.create(("localhost", AGENT_PORT)),
            ContextData(),
            ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
        )

        assert errorIndication is None
        assert errorStatus == 0
        assert len(varBinds) == 1
        assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysDescr.0"
        assert varBinds[0][1].prettyPrint().startswith("PySNMP engine version")
        assert isinstance(varBinds[0][1], OctetString)

        snmpEngine.close_dispatcher()


def test_v1_get_timeout_invalid_target():
    loop = asyncio.get_event_loop()
    snmpEngine = SnmpEngine()

    async def run_get():
        errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
            snmpEngine,
            CommunityData("community_string"),
            await UdpTransportTarget.create(("1.2.3.4", 161), timeout=1, retries=0),
            ContextData(),
            ObjectType(ObjectIdentity("1.3.6.1.4.1.60069.9.1.0")),
        )
        assert isinstance(errorIndication, RequestTimedOut)

    start = datetime.now()
    try:
        loop.run_until_complete(asyncio.wait_for(run_get(), timeout=3))
        end = datetime.now()
        elapsed_time = (end - start).total_seconds()
        assert elapsed_time >= 1 and elapsed_time <= 3  # transport timeout is 1 second
    except asyncio.TimeoutError:
        assert False, "Test case timed out"
    finally:
        snmpEngine.close_dispatcher()


@pytest.mark.asyncio
async def test_v1_get_slow_object():
    async with AgentContextManager(enable_custom_objects=True):
        snmpEngine = SnmpEngine()

        async def run_get():
            errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
                snmpEngine,
                CommunityData("public", mpModel=0),
                await UdpTransportTarget.create(
                    ("localhost", AGENT_PORT),
                    timeout=1,
                    retries=0,  # TODO: why this timeout did not work?
                ),
                ContextData(),
                ObjectType(ObjectIdentity("1.3.6.1.4.1.60069.9.1.0")),
            )
            assert errorIndication is None
            assert errorStatus == 0
            assert len(varBinds) == 1

        start = datetime.now()
        try:
            await asyncio.wait_for(run_get(), timeout=3)
            end = datetime.now()
            elapsed_time = (end - start).total_seconds()
            assert (
                elapsed_time >= 2 and elapsed_time <= 3
            )  # 2 seconds is the delay for the object
        except asyncio.TimeoutError:
            assert False, "Test case timed out"
        finally:
            snmpEngine.close_dispatcher()


@pytest.mark.asyncio
async def test_v1_get_no_access_object():
    async with AgentContextManager(enable_custom_objects=True):
        snmpEngine = SnmpEngine()
        errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
            snmpEngine,
            CommunityData("public", mpModel=0),
            await UdpTransportTarget.create(
                ("localhost", AGENT_PORT), timeout=1, retries=0
            ),
            ContextData(),
            ObjectType(ObjectIdentity("1.3.6.1.4.1.60069.9.3")),
        )
        assert errorIndication is None
        assert errorStatus.prettyPrint() == "noSuchName"  # v1 does not have noAccess
        snmpEngine.close_dispatcher()



import pytest
from pysnmp.hlapi.v3arch.asyncio import *
from pysnmp.smi import builder, compiler, view
from tests.manager_context import MANAGER_PORT, ManagerContextManager


@pytest.mark.asyncio
async def test_send_v3_inform():
    async with ManagerContextManager():
        snmpEngine = SnmpEngine()
        errorIndication, errorStatus, errorIndex, varBinds = await send_notification(
            snmpEngine,
            UsmUserData("usr-md5-des", "authkey1", "privkey1"),
            await UdpTransportTarget.create(("localhost", MANAGER_PORT)),
            ContextData(),
            "inform",
            NotificationType(ObjectIdentity("1.3.6.1.6.3.1.1.5.2"))
            .load_mibs("SNMPv2-MIB")
            .add_varbinds(("1.3.6.1.2.1.1.1.0", OctetString("my system"))),
        )

        assert errorIndication is None
        assert errorStatus == 0
        assert errorIndex == 0
        assert len(varBinds) == 3
        assert (
            varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysUpTime.0"
        )  # IMPORTANT: MIB is needed to resolve this name
        assert varBinds[1][0].prettyPrint() == "SNMPv2-MIB::snmpTrapOID.0"
        assert varBinds[2][0].prettyPrint() == "SNMPv2-MIB::sysDescr.0"
        isinstance(varBinds[0][1], TimeTicks)
        isinstance(varBinds[1][1], ObjectIdentifier)
        isinstance(varBinds[2][1], OctetString)
        snmpEngine.close_dispatcher()



#
# This file is part of pysnmp software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# License: https://www.pysnmp.com/pysnmp/license.html
#
from pyasn1.error import PyAsn1Error
from pysnmp.error import PySnmpError


class SmiError(PySnmpError, PyAsn1Error):
    """Base class for all SNMP SMI-related exceptions."""

    pass


class MibLoadError(SmiError):
    """Base class for all MIB loading exceptions."""

    pass


class MibNotFoundError(MibLoadError):
    """MIB file not found."""

    pass


class MibOperationError(SmiError):
    """Base class for all MIB operation exceptions."""

    def __init__(self, **kwargs):
        """MIB operation error."""
        self.__outArgs = kwargs

    def __str__(self):
        """Return error indication as a string."""
        return f"{self.__class__.__name__}({self.__outArgs})"

    def __getitem__(self, key):
        """Return error indication value by key."""
        return self.__outArgs[key]

    def __contains__(self, key):
        """Return True if key is present in the error indication."""
        return key in self.__outArgs

    def get(self, key, defVal=None):
        """Return error indication value by key."""
        return self.__outArgs.get(key, defVal)

    def keys(self):
        """Return error indication keys."""
        return self.__outArgs.keys()

    def update(self, d):
        """Update error indication."""
        self.__outArgs.update(d)


# Aligned with SNMPv2 PDU error-status values


class TooBigError(MibOperationError):
    """Value too big."""

    pass


class NoSuchNameError(MibOperationError):
    """No such name."""

    pass


class BadValueError(MibOperationError):
    """Bad value."""

    pass


class ReadOnlyError(MibOperationError):
    """Read-only."""

    pass


class GenError(MibOperationError):
    """General error."""

    pass


class NoAccessError(MibOperationError):
    """No access."""

    pass


class WrongTypeError(MibOperationError):
    """Wrong type."""

    pass


class WrongLengthError(MibOperationError):
    """Wrong length."""

    pass


class WrongEncodingError(MibOperationError):
    """Wrong encoding."""

    pass


class WrongValueError(MibOperationError):
    """Wrong value."""

    pass


class NoCreationError(MibOperationError):
    """No creation."""

    pass


class InconsistentValueError(MibOperationError):
    """Inconsistent value."""

    pass


class ResourceUnavailableError(MibOperationError):
    """Resource unavailable."""

    pass


class CommitFailedError(MibOperationError):
    """Commit failed."""

    pass


class UndoFailedError(MibOperationError):
    """Undo failed."""

    pass


class AuthorizationError(MibOperationError):
    """Authorization error."""

    pass


class NotWritableError(MibOperationError):
    """Not writable."""

    pass


class InconsistentNameError(MibOperationError):
    """Inconsistent name."""

    pass


# Aligned with SNMPv2 PDU exceptions or error-status values


class NoSuchObjectError(NoSuchNameError):
    """No such object."""

    pass


class NoSuchInstanceError(NoSuchNameError):
    """No such instance."""

    pass


class EndOfMibViewError(NoSuchNameError):
    """End of MIB view."""

    pass


# SNMP table management exceptions


class TableRowManagement(MibOperationError):
    """Base class for all SNMP table row management exceptions."""

    pass


class RowCreationWanted(TableRowManagement):
    """Row creation wanted."""

    pass


class RowDestructionWanted(TableRowManagement):
    """Row destruction wanted."""

    pass



import asyncio
import pytest


from pysnmp.hlapi.v3arch.asyncio import *
from pysnmp.proto.api import v2c
from tests.manager_context import MANAGER_PORT, ManagerContextManager


@pytest.mark.asyncio
async def test_send_trap_enterprise_specific():
    async with ManagerContextManager() as (_, message_count):
        snmpEngine = SnmpEngine()
        errorIndication, errorStatus, errorIndex, varBinds = await send_notification(
            snmpEngine,
            CommunityData("public", mpModel=0),
            await UdpTransportTarget.create(("localhost", MANAGER_PORT)),
            ContextData(),
            "trap",
            NotificationType(
                ObjectIdentity("1.3.6.1.4.1.20408.4.1.1.2.432")
            ).add_varbinds(
                (v2c.apiTrapPDU.sysUpTime, TimeTicks(12345)),
                ("1.3.6.1.2.1.1.1.0", OctetString("my system")),
                (v2c.apiTrapPDU.snmpTrapAddress, IpAddress("127.0.0.1")),
                (
                    ObjectIdentity("SNMPv2-MIB", "snmpTrapOID", 0),
                    ObjectIdentifier("1.3.6.1.4.1.20408.4.1.1.2.432"),
                ),
            ),
        )

        snmpEngine.close_dispatcher()
        await asyncio.sleep(1)
        assert message_count == [1]


@pytest.mark.asyncio
async def test_send_trap_generic():
    async with ManagerContextManager() as (_, message_count):
        snmpEngine = SnmpEngine()
        errorIndication, errorStatus, errorIndex, varBinds = await send_notification(
            snmpEngine,
            CommunityData("public", mpModel=0),
            await UdpTransportTarget.create(("localhost", MANAGER_PORT)),
            ContextData(),
            "trap",
            NotificationType(ObjectIdentity("1.3.6.1.6.3.1.1.5.2"))
            .load_mibs("SNMPv2-MIB")
            .add_varbinds(
                (
                    "1.3.6.1.6.3.1.1.4.3.0",
                    "1.3.6.1.4.1.20408.4.1.1.2",
                ),  # IMPORTANT: MIB is needed to resolve str to correct type.
                ("1.3.6.1.2.1.1.1.0", OctetString("my system")),
            ),
        )

        snmpEngine.close_dispatcher()
        await asyncio.sleep(1)
        assert message_count == [1]


@pytest.mark.asyncio
async def test_send_trap_custom_mib():
    async with ManagerContextManager() as (_, message_count):
        snmpEngine = SnmpEngine()
        mibBuilder = snmpEngine.get_mib_builder()
        (sysUpTime,) = mibBuilder.import_symbols("__SNMPv2-MIB", "sysUpTime")
        sysUpTime.syntax = TimeTicks(12345)

        errorIndication, errorStatus, errorIndex, varBinds = await send_notification(
            snmpEngine,
            CommunityData("public", mpModel=0),
            await UdpTransportTarget.create(("localhost", MANAGER_PORT)),
            ContextData(),
            "trap",
            NotificationType(
                ObjectIdentity("NET-SNMP-EXAMPLES-MIB", "netSnmpExampleNotification")
            ).add_varbinds(
                ObjectType(
                    ObjectIdentity(
                        "NET-SNMP-EXAMPLES-MIB", "netSnmpExampleHeartbeatRate"
                    ),
                    1,
                )
            ),
        )

        snmpEngine.close_dispatcher()
        await asyncio.sleep(1)
        assert message_count == [1]



import pytest
from pysnmp.hlapi.v1arch.asyncio import *
from tests.agent_context import AGENT_PORT, AgentContextManager


total_count = 212  # 267


@pytest.mark.asyncio
async def test_v2_walk():  # some agents have different v2 GET NEXT behavior
    async with AgentContextManager():
        snmpDispatcher = SnmpDispatcher()
        objects = walk_cmd(
            snmpDispatcher,
            CommunityData("public"),
            await UdpTransportTarget.create(("localhost", AGENT_PORT)),
            ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
        )

        objects_list = [item async for item in objects]

        errorIndication, errorStatus, errorIndex, varBinds = objects_list[0]

        assert errorIndication is None
        assert errorStatus == 0
        assert len(varBinds) == 1
        assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysObjectID.0"

        errorIndication, errorStatus, errorIndex, varBinds = objects_list[1]

        assert errorIndication is None
        assert errorStatus == 0
        assert len(varBinds) == 1
        assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysUpTime.0"

        assert len(objects_list) == total_count

        snmpDispatcher.transport_dispatcher.close_dispatcher()


@pytest.mark.asyncio
async def test_v2_walk_subtree():
    async with AgentContextManager():
        snmpDispatcher = SnmpDispatcher()
        objects = walk_cmd(
            snmpDispatcher,
            CommunityData("public"),
            await UdpTransportTarget.create(("localhost", AGENT_PORT)),
            ObjectType(ObjectIdentity("SNMPv2-MIB", "system")),
            lexicographicMode=False,
        )

        objects_list = [item async for item in objects]

        errorIndication, errorStatus, errorIndex, varBinds = objects_list[0]

        assert errorIndication is None
        assert errorStatus == 0
        assert len(varBinds) == 1
        assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysDescr.0"

        errorIndication, errorStatus, errorIndex, varBinds = objects_list[1]

        assert errorIndication is None
        assert errorStatus == 0
        assert len(varBinds) == 1
        assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysObjectID.0"

        assert len(objects_list) == 8

        snmpDispatcher.transport_dispatcher.close_dispatcher()



import pytest
from pysnmp.hlapi.v3arch.asyncio import *
from tests.agent_context import AGENT_PORT, AgentContextManager


@pytest.mark.asyncio
async def test_v1_set():
    async with AgentContextManager():
        snmpEngine = SnmpEngine()
        errorIndication, errorStatus, errorIndex, varBinds = await set_cmd(
            snmpEngine,
            CommunityData("public", mpModel=0),
            await UdpTransportTarget.create(("localhost", AGENT_PORT)),
            ContextData(),
            ObjectType(ObjectIdentity("SNMPv2-MIB", "sysLocation", 0), "Shanghai"),
        )

        assert errorIndication is None
        assert errorStatus == 0
        assert len(varBinds) == 1
        assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysLocation.0"
        assert varBinds[0][1].prettyPrint() == "Shanghai"
        assert isinstance(varBinds[0][1], OctetString)

        snmpEngine.close_dispatcher()


@pytest.mark.asyncio
async def test_v1_set_table_creation():
    async with AgentContextManager(enable_table_creation=True):
        snmpEngine = SnmpEngine()

        # Perform a SNMP walk to get all object counts
        objects = walk_cmd(
            snmpEngine,
            CommunityData("public", mpModel=0),
            await UdpTransportTarget.create(("localhost", AGENT_PORT)),
            ContextData(),
            ObjectType(ObjectIdentity("1.3.6")),
        )

        objects_list = [item async for item in objects]
        assert len(objects_list) > 0

        object_counts = len(objects_list)

        errorIndication, errorStatus, errorIndex, varBinds = await set_cmd(
            snmpEngine,
            CommunityData("public", mpModel=0),
            await UdpTransportTarget.create(("localhost", AGENT_PORT)),
            ContextData(),
            ObjectType(
                ObjectIdentity("1.3.6.6.1.5.2.97.98.99"), OctetString("My value")
            ),
        )

        assert errorIndication is None
        assert errorStatus == 0
        assert len(varBinds) == 1
        assert varBinds[0][0].prettyPrint() == "SNMPv2-SMI::dod.6.1.5.2.97.98.99"
        assert varBinds[0][1].prettyPrint() == "My value"
        assert isinstance(varBinds[0][1], OctetString)

        errorIndication, errorStatus, errorIndex, varBinds = await set_cmd(
            snmpEngine,
            CommunityData("public", mpModel=0),
            await UdpTransportTarget.create(("localhost", AGENT_PORT)),
            ContextData(),
            ObjectType(ObjectIdentity("1.3.6.6.1.5.4.97.98.99"), Integer(4)),
        )

        assert errorIndication is None
        assert errorStatus == 0
        assert len(varBinds) == 1
        assert varBinds[0][0].prettyPrint() == "SNMPv2-SMI::dod.6.1.5.4.97.98.99"
        assert varBinds[0][1].prettyPrint() == "1"
        # assert isinstance(varBinds[0][1], Integer)

        # Perform a SNMP walk to get all object counts
        objects = walk_cmd(
            snmpEngine,
            CommunityData("public", mpModel=0),
            await UdpTransportTarget.create(("localhost", AGENT_PORT)),
            ContextData(),
            ObjectType(ObjectIdentity("1.3.6")),
        )

        objects_list = [item async for item in objects]
        assert len(objects_list) > 0

        assert len(objects_list) == object_counts + 4



import pytest
from pysnmp.hlapi.v1arch.asyncio import *
from tests.agent_context import AGENT_PORT, AgentContextManager


@pytest.mark.asyncio
async def test_v2_get():
    async with AgentContextManager():
        snmpDispatcher = SnmpDispatcher()
        errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
            snmpDispatcher,
            CommunityData("public"),
            await UdpTransportTarget.create(("localhost", AGENT_PORT)),
            ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
        )

        assert errorIndication is None
        assert errorStatus == 0
        assert len(varBinds) == 1
        assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysDescr.0"
        assert varBinds[0][1].prettyPrint().startswith("PySNMP engine version")
        assert isinstance(varBinds[0][1], OctetString)

        snmpDispatcher.transport_dispatcher.close_dispatcher()


@pytest.mark.asyncio
async def test_v2_get_no_access_object():
    async with AgentContextManager(enable_custom_objects=True):
        snmpDispatcher = SnmpDispatcher()
        errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
            snmpDispatcher,
            CommunityData("public"),
            await UdpTransportTarget.create(
                ("localhost", AGENT_PORT), timeout=1, retries=0
            ),
            ObjectType(ObjectIdentity("1.3.6.1.4.1.60069.9.3")),
        )

        assert errorIndication is None
        assert errorStatus.prettyPrint() == "noAccess"  # v2c and v3 use noAccess
        snmpDispatcher.transport_dispatcher.close_dispatcher()


@pytest.mark.asyncio
async def test_v2_get_legacy_object():
    async with AgentContextManager(enable_custom_objects=True):
        snmpDispatcher = SnmpDispatcher()
        errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
            snmpDispatcher,
            CommunityData("public"),
            await UdpTransportTarget.create(
                ("localhost", AGENT_PORT), timeout=1, retries=0
            ),
            ObjectType(ObjectIdentity("1.3.6.1.4.1.60069.9.4")),
        )

        assert errorIndication is None
        assert (
            errorStatus.prettyPrint() == "noAccess"
        )  # PySMI <1.3.0 generates such objects
        snmpDispatcher.transport_dispatcher.close_dispatcher()



import pytest
from pysnmp.hlapi.v3arch.asyncio import *
from pysnmp.proto.errind import UnknownUserName, WrongDigest
from tests.agent_context import AGENT_PORT, AgentContextManager


@pytest.mark.asyncio
async def test_usm_sha_none():
    async with AgentContextManager():
        snmpEngine = SnmpEngine()
        authData = UsmUserData(
            "usr-sha-none",
            "authkey1",
            authProtocol=USM_AUTH_HMAC96_SHA,
        )
        errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
            snmpEngine,
            authData,
            await UdpTransportTarget.create(("localhost", AGENT_PORT), retries=0),
            ContextData(),
            ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
        )

        assert errorIndication is None
        assert errorStatus == 0
        assert len(varBinds) == 1
        assert varBinds[0][0].prettyPrint() == "SNMPv2-MIB::sysDescr.0"
        isinstance(varBinds[0][1], OctetString)

        snmpEngine.close_dispatcher()


@pytest.mark.asyncio
async def test_usm_sha_none_wrong_auth():
    async with AgentContextManager():
        snmpEngine = SnmpEngine()
        authData = UsmUserData(
            "usr-sha-none",
            "authkey1",
            authProtocol=USM_AUTH_HMAC96_MD5,  # wrongly use usmHMACMD5AuthProtocol
        )
        errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
            snmpEngine,
            authData,
            await UdpTransportTarget.create(("localhost", AGENT_PORT), retries=0),
            ContextData(),
            ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
        )

        assert isinstance(errorIndication, WrongDigest)
        assert str(errorIndication) == "Wrong SNMP PDU digest"

        snmpEngine.close_dispatcher()


@pytest.mark.asyncio
async def test_usm_sha_none_wrong_user():
    async with AgentContextManager():
        snmpEngine = SnmpEngine()
        authData = UsmUserData(
            "usr-sha-none-not-exist",
            "authkey1",
            authProtocol=USM_AUTH_HMAC96_SHA,
        )
        errorIndication, errorStatus, errorIndex, varBinds = await get_cmd(
            snmpEngine,
            authData,
            await UdpTransportTarget.create(("localhost", AGENT_PORT), retries=0),
            ContextData(),
            ObjectType(ObjectIdentity("SNMPv2-MIB", "sysDescr", 0)),
        )

        assert isinstance(errorIndication, UnknownUserName)
        assert str(errorIndication) == "Unknown USM user"

        snmpEngine.close_dispatcher()
